;;; **********************************************************************
;;; Copyright (C) 2005-2006 Todd Ingall, Rick Taube
;;;
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the Lisp Lesser Gnu Public License.
;;; See http://www.cliki.net/LLGPL for the text of this agreement.
;;; **********************************************************************

;;; $Name: rel-2_0_1 $
;;; $Revision: 1.7 $
;;; $Date: 2007/01/08 23:56:06 $

#-asdf 
(error "Portmidi requires ASDF to load.")

(in-package :cl-user)

;; if CFFI isn't already loaded or on *central-registry*, search for
;; it first under this directory then under the parent directory of
;; this directory, else signal an error.

(unless (or (asdf:find-system :cffi nil)
            (asdf:system-definition-pathname :cffi))
  (flet ((cffi-probe (dir)
           (let ((asd
                  (first (directory
                          (make-pathname :name "cffi" :type "asd"
                                         :directory (append 
                                                     (pathname-directory dir)
                                                     (list :wild))
                                         :defaults dir)))))
             (if asd (push asd asdf:*central-registry*) nil))))
    (let* ((thisdir (make-pathname :type nil :name nil
                                   :defaults (truename *load-pathname*)))
           (rootdir (make-pathname :directory
                                   (butlast (pathname-directory thisdir))
                                   :defaults thisdir)))
      (or (cffi-probe thisdir)
          (cffi-probe rootdir)
          (error "Atempt to load portmidi without CFFI loaded.")))))

; (progn (require :asdf) (push "/Lisp/portmidi/" asdf::*central-registry*) (asdf::oos 'asdf::load-op :portmidi))

(defclass lib-dependant-file (asdf:cl-source-file)
  ((lib :initarg :lib)
   (path :initarg :path :initform ())
   (if-not-exist :initarg :if-not-exist :initform :error)))
  
(defun find-component-lib (c)
  (let ((lib (slot-value c 'lib))
	(ext #+darwin "dylib" #+linux "so")
	(dir (slot-value c 'path))
	(fil (asdf:component-pathname c)))
    (loop for p in dir
       for d = (if (equal p "./")
		   (pathname-directory fil)
		   (pathname-directory p))
       thereis (probe-file (make-pathname :name lib :type ext
					  :directory d :defaults fil)))))

(defmethod asdf:perform ((op asdf:compile-op) (c lib-dependant-file))
  (if (find-component-lib c) (call-next-method) nil))

(defmethod asdf:perform ((op asdf:load-op) (c lib-dependant-file))
  (let ((l (find-component-lib c)))
    ;;(print (LIST :HERE C (find-component-lib c)))
    (if (null l)
	(ecase (slot-value c 'if-not-exist)
	  (( :error )
	   (error "System ~a: required library ~s could not be found in: ~s"
		  (asdf:component-name (asdf:component-system c))
		  (slot-value c 'lib)
		  (slot-value c 'path))
	   nil)
	  (( :warn )
	   (warn "System ~a: required library ~s could not be found in: ~s"
		 (asdf:component-name (asdf:component-system c))
		 (slot-value c 'lib)
		 (slot-value c 'path))
	   nil)
	  (( :omit :skip )
	   nil))
	(call-next-method))))

(asdf:defsystem :portmidi
    :description "CFFI Interface to Portmidi"
    :author "Todd Ingalls, Rick Taube>"
    :licence "LLGPL"
    :depends-on (cffi)
    :serial t
    :components (#||(:lib-dependant-file "cffi-portmidi"
				      :lib "libportmidi"
				      :path ("/usr/local/lib/" "/usr/lib64/" "./")
				      :if-not-exist :error)||#
		 (:file "cffi-portmidi")
		 (:lib-dependant-file "cffi-recv" :lib "libpmrecv"
				      :path ("./")
				      :if-not-exist :omit)
		 #+cm 
		 (:lib-dependant-file "cm-recv" :lib "libpmrecv"
				      :path ("./")
				      :if-not-exist :omit)
		 )
    )

